#!/usr/bin/env python3

import shutil
try:
    import sh
except:
    import pbs as sh
gcc = sh.Command("g++")
import time

tests = sh.glob("tests/*/?")

PROB = "power"
INPUT = "{}.in".format(PROB)
OUTPUT = "{}.out".format(PROB)

SOLUTIONS = [
    "power_ak_n2_cpp2.cpp",
    "power_ak.java",
    "power_ak_treeset.java",
    "power_ak_py.py",
    "power_ak.cpp",
    "power_ak_wa.java"
#    "power_ak_wa.java"
]

outname = lambda s: s.rsplit(".", 1)[0]

def compile(fname):
    if fname.endswith(".java"):
        sh.javac(fname)
    elif fname.endswith(".cpp"):
        gcc(fname, "-std=c++11", "-O2", "-o", outname(fname))

def run(fname):
    if fname.endswith(".java"):
        sh.java(outname(fname))
    elif fname.endswith(".cpp"):
        sh.Command("./" + outname(fname))()
    else:
        sh.python3(fname)

def run_on_test(sol, test):
    shutil.copy(test, INPUT)
    start = time.time()
    run(sol)
    end = time.time()
    shutil.copy(OUTPUT, "{}.{}.a".format(test, sol.split("_", 1)[1]))
    print("{} on {}: {} sec".format(sol, test, end - start))
    return (end - start, open(OUTPUT).read())


tests = sh.glob("tests/*/??")
print(tests)
total = {}
answers = {}
for sol in SOLUTIONS:
    compile(sol)
    times = {}
    for t in tests:
        runtime, output = run_on_test(sol, t)
        times[t] = runtime
        answers[(sol, t)] = output
    total[sol] = max(times.values())
    print("{}: max {}".format(sol, max(times.values())))

print(total)
for t in tests:
    ans = {sol: answers[(sol, t)] for sol in SOLUTIONS}
    if len(set(ans.values())) != 1:
        print("FAIL on {}: {}".format(t, ans))
