#!/bin/bash

problem=qual
author=av

ulimit -s 65536

echo Compiling checker
g++ -std=c++11 -O2 -I../../../lib -o check.exe check.cpp

echo Compiling validator
g++ -std=c++11 -O2  -I../../../lib -o validate_subtask1.exe validate_subtask1.cpp
g++ -std=c++11 -O2  -I../../../lib -o validate_subtask2.exe validate_subtask2.cpp
g++ -std=c++11 -O2  -I../../../lib -o validate_subtask3.exe validate_subtask3.cpp
g++ -std=c++11 -O2  -I../../../lib -o validate_subtask4.exe validate_subtask4.cpp

echo Generating tests
rm -rf preliminary
rm -rf tests
pushd src
javac TestGen.java
java TestGen
popd

echo Compiling main solution
pushd main
g++ -std=c++11 -O2  -Wall -Wextra -Wconversion -Werror -Wno-unused-result -I../../../lib -o ../${problem}_${author}.exe ${problem}_${author}.cpp
popd

echo Generating answers
for i in preliminary tests/subtask1 tests/subtask2 tests/subtask3 tests/subtask4
do
    for j in $i/*
    do
        echo Copying input for test $j
        cp $j ${problem}.in > /dev/null

        if [ "$i" == "tests/subtask1" ]; then
            ./validate_subtask1.exe < ${problem}.in
            if [ $? -ne 0 ]; then
                echo Validator rejects test $j
                exit
            fi
        fi
        if [ "$i" == "tests/subtask2" ]; then
            ./validate_subtask2.exe < ${problem}.in
            if [ $? -ne 0 ]; then
                echo Validator rejects test $j
                exit
            fi
        fi
        if [ "$i" == "tests/subtask3" ]; then
            ./validate_subtask3.exe < ${problem}.in
            if [ $? -ne 0 ]; then
                echo Validator rejects test $j
                exit
            fi
        fi
        if [ "$i" == "tests/subtask4" ]; then
            ./validate_subtask4.exe < ${problem}.in
            if [ $? -ne 0 ]; then
                echo Validator rejects test $j
                exit
            fi
        fi
        echo Validator accepts test $j

        echo Running solution on test $j
        ./${problem}_${author}.exe
        if [ $? -ne 0 ]; then
            echo Solution failed on test $j
            exit
        fi

        echo Running checker on test $j
		./check.exe $j ${problem}.out ${problem}.out
        if [ $? -ne 0 ]; then
            echo Checker rejects main solution output on test $j
            exit
        fi

        echo Saving answer for test $j
        cp ${problem}.out ${j}.a > /dev/null
    done
done

